<?php
/**
 * DBCart 商城系统
 *
 * ==========================================================================
 * @link      https://www.dbcart.cn/
 * @copyright 北京珑大钜商科技有限公司，并保留所有权利。
 * @license   https://www.dbcart.cn/license.html License
 * ==========================================================================
 *
 * @author    静静的风 <baron@loongdom.cn>
 *
 */

namespace Admin\Controller;

use Admin\Data\Common;
use Admin\Data\Config;
use Admin\Entity\Currency;
use Admin\Entity\CurrencyContent;
use Admin\Form\CurrencyForm;
use Admin\Service\CurrencyContentManager;
use Admin\Service\CurrencyManager;
use Doctrine\ORM\EntityManager;
use Laminas\Http\Client;
use Laminas\Mvc\I18n\Translator;
use Laminas\View\Model\JsonModel;

class CurrencyController extends AdminActionController
{
    private $translator;
    private $entityManager;
    private $currencyManager;
    private $currencyContentManager;

    public function __construct(
        Translator $translator,
        EntityManager $entityManager,
        CurrencyManager $currencyManager,
        CurrencyContentManager $currencyContentManager
    )
    {
        $this->translator       = $translator;
        $this->entityManager    = $entityManager;
        $this->currencyManager  = $currencyManager;
        $this->currencyContentManager = $currencyContentManager;
    }

    /**
     * 货币列表
     * @return array|\Laminas\View\Model\ViewModel
     */
    public function indexAction()
    {
        $currencyList = $this->entityManager->getRepository(Currency::class)->findCurrencyList();

        return ['currencyList' => $currencyList];
    }

    /**
     * 添加货币
     * @return array|\Laminas\Http\Response
     * @throws \Doctrine\ORM\ORMException
     * @throws \Doctrine\ORM\OptimisticLockException
     */
    public function addAction()
    {
        $form = new CurrencyForm();
        $form->get('currencyCode')->setValueOptions($this->adminCommon()->currencyArray());
        $form->get('currencyState')->setValueOptions(Common::state($this->translator));
        $form->get('currencyDecimal')->setValueOptions([0 => 0, 2 => 2]);

        if($this->getRequest()->isPost()) {
            $data = $this->params()->fromPost();
            $form->setData($data);
            if($form->isValid()) {
                $data = $form->getData();

                $numberFormatter = new \NumberFormatter(Config::SHOP_CURRENCY_LOCALE . '@currency=' . $data['currencyCode'], \NumberFormatter::CURRENCY);
                $data['currencySymbol'] = $numberFormatter->getSymbol(\NumberFormatter::CURRENCY_SYMBOL);

                $data['currencyName'] = $data['currencyName'.SHOP_FONT_LANGUAGE];
                $currency = $this->currencyManager->addCurrency($data);
                $this->currencyContentManager->addCurrencyContent($data, $currency->getCurrencyId());
                $this->createConfig()->createCurrency();

                $this->adminCommon()->addOperLog(sprintf($this->translator->translate('货币 %s 添加成功!'), $data['currencyName']), $this->translator->translate('货币设置'));

                return $this->redirect()->toRoute('currency');
            }
        }

        $defaultCurrency = $this->entityManager->getRepository(Currency::class)->findOneBy(['currencyType' => 1]);

        return ['form' => $form, 'defaultCurrency' => $defaultCurrency];
    }

    /**
     * 编辑货币
     * @return array|\Laminas\Http\Response
     * @throws \Doctrine\ORM\ORMException
     * @throws \Doctrine\ORM\OptimisticLockException
     */
    public function editAction()
    {
        $currencyId = (int) $this->params()->fromRoute('id', -1);

        $currencyInfo = $this->entityManager->getRepository(Currency::class)->findOneByCurrencyId($currencyId);
        if($currencyInfo == null) {
            $this->flashMessenger()->addWarningMessage($this->translator->translate('该货币信息不存在!'));
            return $this->redirect()->toRoute('currency');
        }

        $form = new CurrencyForm();
        $form->get('currencyCode')->setValueOptions($this->adminCommon()->currencyArray());
        $form->get('currencyState')->setValueOptions(Common::state($this->translator));
        $form->get('currencyDecimal')->setValueOptions([0 => 0, 2 => 2]);

        if($this->getRequest()->isPost()) {
            $data = $this->params()->fromPost();
            $form->setData($data);
            if ($form->isValid()) {
                $data = $form->getData();

                $numberFormatter = new \NumberFormatter(Config::SHOP_CURRENCY_LOCALE . '@currency=' . $data['currencyCode'], \NumberFormatter::CURRENCY);
                $data['currencySymbol'] = $numberFormatter->getSymbol(\NumberFormatter::CURRENCY_SYMBOL);

                $data['currencyName'] = $data['currencyName'.SHOP_FONT_LANGUAGE];
                $this->currencyManager->editCurrency($data, $currencyInfo);
                $this->currencyContentManager->editCurrencyContent($data, $currencyId);
                $this->createConfig()->createCurrency();

                $this->adminCommon()->addOperLog(sprintf($this->translator->translate('货币 %s 编辑成功!'), $data['currencyName'.SHOP_FONT_LANGUAGE]), $this->translator->translate('货币设置'));

                return $this->redirect()->toRoute('currency');
            }
        } else $form->setData($currencyInfo->valuesArray());

        $currencyContentList = $this->entityManager->getRepository(CurrencyContent::class)->findBy(['currencyId' => $currencyId]);
        foreach ($currencyContentList as $contentValue) {
            if (!in_array($contentValue->getLanguageCode(), SHOP_LANGUAGE)) continue;
            $form->get('currencyName'.$contentValue->getLanguageCode())->setValue($contentValue->getCurrencyName());
        }

        $defaultCurrency = $this->entityManager->getRepository(Currency::class)->findOneBy(['currencyType' => 1]);

        return ['form' => $form, 'currency' => $currencyInfo, 'defaultCurrency' => $defaultCurrency];
    }

    /**
     * 删除货币
     * @return \Laminas\Http\Response
     * @throws \Doctrine\ORM\ORMException
     * @throws \Doctrine\ORM\OptimisticLockException
     */
    public function deleteAction()
    {
        if(!$this->adminCommon()->validatorCsrf()) return $this->redirect()->toRoute('currency');

        $currencyId = (int) $this->params()->fromRoute('id', -1);
        if($currencyId <= 1) {
            $this->flashMessenger()->addErrorMessage($this->translator->translate('不能删除基础货币!'));
            return $this->redirect()->toRoute('currency');
        }

        $currencyInfo = $this->entityManager->getRepository(Currency::class)->findOneByCurrencyId($currencyId);
        if($currencyInfo == null) {
            $this->flashMessenger()->addWarningMessage($this->translator->translate('该货币信息不存在!'));
            return $this->redirect()->toRoute('currency');
        }

        $this->currencyManager->deleteCurrency($currencyInfo);
        $this->currencyContentManager->deleteCurrencyContent($currencyId);
        $this->createConfig()->createCurrency();

        $this->adminCommon()->addOperLog(sprintf($this->translator->translate('货币 %s 删除成功!'), $currencyInfo->getCurrencyName()), $this->translator->translate('货币设置'));

        return $this->adminCommon()->toReferer();
    }

    /**
     * 更新货币汇率
     * @return JsonModel
     */
    public function updateRateAction(): JsonModel
    {
        $currencyFile = 'data/moduleData/System/currencyConfig.php';
        if (!file_exists($currencyFile)) return new JsonModel(['status' => 0, 'message' => $this->translator->translate('货币配置文件不存在!')]);

        $currencyConfig = include $currencyFile;
        if (count($currencyConfig) <= 2) return new JsonModel(['status' => 0, 'message' => $this->translator->translate('只有基础货币，无需更新货币汇率!')]);

        $client = new Client('https://open.er-api.com/v6/latest/'.$currencyConfig['default']['code'], [
            'adapter'       => 'Laminas\Http\Client\Adapter\Curl',
            'curloptions'   => [CURLOPT_SSL_VERIFYPEER => false]
        ]);
        $client->setHeaders(['Accept' => 'application/json']);
        $client->setMethod('GET');
        try {
            $response = $client->send();
            if ($response->isSuccess()) {
                $result = json_decode($response->getBody(), true);
                foreach ($currencyConfig as $key => $value) {
                    if ($key == 'default') continue;
                    if (isset($result['rates'][$key]) && $result['rates'][$key] != $value['rate']) {
                        $currencyInfo = $this->entityManager->getRepository(Currency::class)->findOneBy(['currencyCode' => $key]);
                        if ($currencyInfo) {
                            $this->currencyManager->editCurrency(['currencyRate' => number_format($result['rates'][$key], 4, '.', '')], $currencyInfo);
                        }
                    }
                }
                $this->createConfig()->createCurrency();

                return new JsonModel(['status' => 1, 'message' => $this->translator->translate('货币汇率更新成功!')]);
            } else {
                return new JsonModel(['status' => 0, 'message' => $this->translator->translate('货币汇率更新失败!')]);
            }
        } catch (\Exception$e) {
            return new JsonModel(['status' => 0, 'message' => $this->translator->translate('货币汇率更新失败!')]);
        }
    }
}